import matplotlib.pyplot as plt
import numpy as np

from mpl_toolkits.mplot3d import Axes3D
from random import random, randrange

"""
Simpler functions
"""

N = 25
R = 0.02
Tmax = 10.0
Tmin = 1e-3
tau = 1e4

def mag(x):
    """
    Function to calculate the magnitude of a vector in 
    two directions.
    """
    return np.sqrt(x[0]**2+x[1]**2)

def distance():
    """
    Function to calculate the total length of the tour.
    How far has the salesman traveled?
    """
    s = 0.0
    for i in range(N):
        s += mag(r[i+1]-r[i])
    return s

# Choose N city locations and calculate the initial distance
r = np.empty([N+1,2], float)
for i in range(N):
    r[i,0] = random()
    r[i,1] = random()
r[N] = r[0]
D = distance()

# Set up the graphics
u, v = np.mgrid[0:2*np.pi:20j, 0:np.pi:10j]
x = np.cos(r[-1])
y = np.sin(r[-1])
z = np.cos(R)
ax.plot_wireframe(x, y, z, color="r")

# Main loop
t = 0
T = Tmax
while T>Tmin:

    # Cooling
    t += 1
    T = Tmax*np.exp(-t/tau)

    # Update the visualization every 100 moves
    if t%100==0:
        l.pos = r

    # Choose two cities to swap and make sure they are distinct
    i,j = randrange(1,N),randrange(1,N)
    while i==j:
        i,j = randrange(1,N),randrange(1,N)

    # Swap them and calculate the change in distance
    oldD = D
    r[i,0],r[j,0] = r[j,0],r[i,0]
    r[i,1],r[j,1] = r[j,1],r[i,1]
    D = distance()
    deltaD = D - oldD

    # If the move is rejected, swap them back again
    if random() > np.exp(-deltaD/T):
        r[i,0],r[j,0] = r[j,0],r[i,0]
        r[i,1],r[j,1] = r[j,1],r[i,1]
        D = oldD

plt.figure()
